(************************************************************************)
(*         *   The Coq Proof Assistant / The Coq Development Team       *)
(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
(* <O___,, *       (see CREDITS file for the list of authors)           *)
(*   \VV/  **************************************************************)
(*    //   *    This file is distributed under the terms of the         *)
(*         *     GNU Lesser General Public License Version 2.1          *)
(*         *     (see LICENSE file for the text of the license)         *)
(************************************************************************)

open Hashtbl

module type PHashtable = sig
  type 'a t
  type key

  val open_in : string -> 'a t
  (** [open_in f] rebuilds a table from the records stored in file [f].
        As marshaling is not type-safe, it might segfault.
    *)

  val find : 'a t -> key -> 'a
  (** find has the specification of Hashtable.find *)

  val add : 'a t -> key -> 'a -> unit
  (** [add tbl key elem] adds the binding [key] [elem] to the table [tbl].
        (and writes the binding to the file associated with [tbl].)
        If [key] is already bound, raises KeyAlreadyBound *)

  val memo : string -> (key -> 'a) -> key -> 'a
  (** [memo cache f] returns a memo function for [f] using file [cache] as persistent table.
          Note that the cache will only be loaded when the function is used for the first time *)

  val memo_cond : string -> (key -> bool) -> (key -> 'a) -> key -> 'a
  (** [memo cache cond f] only use the cache if [cond k] holds for the key [k].  *)
end

module PHashtable (Key : HashedType) : PHashtable with type key = Key.t
